/*
 * @author Jarek Szymczak
 */
package gssim.db.find;

import java.sql.Timestamp;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Map;

/**
 * The Class Finder. Get methods are responsible for setting with type checked
 * searching criteria. There is also possible unchecked criteria adding
 * (setCriteria). Class also contains useful methods for searching intervals
 * getMin...(), getMax() which are returnig object (if exists) or min / max
 * value of the variable type (in database).
 * 
 * @author Jarek Szymczak
 */
public class Finder {

	Map<String, Object> map = new HashMap<String, Object>();

	private Finder() {
	};

	public static Finder getTaskFinder(Integer expId, Integer jobId,
			Integer taskId, Integer procSetId) {
		return getTaskFinder(expId, jobId, taskId, procSetId, null, null, null,
				null);
	}

	public static Finder getTaskFinder(Integer expId, Integer jobId,
			Integer taskId, Integer procSetId, Timestamp minExecutionStartDate,
			Timestamp maxExecutionStartDate, Timestamp minExecutionFinishDate,
			Timestamp maxExecutionFinishDate) {

		Finder finder = new Finder();

		finder.setCriteria("expId", expId);
		finder.setCriteria("jobId", jobId);
		finder.setCriteria("taskId", taskId);
		finder.setCriteria("procSetId", procSetId);
		finder.setCriteria("minExecutionStartDate", minExecutionStartDate);
		finder.setCriteria("maxExecutionStartDate", maxExecutionStartDate);
		finder.setCriteria("minExecutionFinishDate", minExecutionFinishDate);
		finder.setCriteria("maxExecutionFinishDate", maxExecutionFinishDate);

		return finder;
	}

	public static Finder getUserFinder(String dn) {

		Finder finder = new Finder();

		finder.setCriteria("dn", dn);

		return finder;
	}

	public static Finder getAllocationFinder(Integer taskId, Integer unitId) {
		return getReservationFinder(taskId, unitId);
	}

	public static Finder getReservationFinder(Integer taskId, Integer unitId) {

		Finder finder = new Finder();

		finder.setCriteria("taskId", taskId);
		finder.setCriteria("unitId", unitId);

		return finder;
	}

	public static Finder getExperimentFinder(Integer userId,
			Timestamp minCommissionDate, Timestamp maxCommissionDate,
			String name, String description, String status, Integer metaexpId) {

		Finder finder = new Finder();

		finder.setCriteria("userId", userId);
		finder.setCriteria("minCommissionDate", minCommissionDate);
		finder.setCriteria("maxCommissionDate", maxCommissionDate);
		finder.setCriteria("name", name);
		finder.setCriteria("description", description);
		finder.setCriteria("status", status);
		finder.setCriteria("metaexpId", metaexpId);

		return finder;
	}

	public static Finder getMetaexperimentFinder(String name) {

		Finder finder = new Finder();

		finder.setCriteria("name", name);

		return finder;
	}

	public static Finder getResourceFinder(Integer expId, String name,
			String type) {

		Finder finder = new Finder();

		finder.setCriteria("expId", expId);
		finder.setCriteria("name", name);
		finder.setCriteria("type", type);

		return finder;
	}
	
	public static Finder getPluginFinder(Integer id, String name, Boolean global) {

		Finder finder = new Finder();

		finder.setCriteria("id", id);
		finder.setCriteria("name", name);
		finder.setCriteria("global", global);

		return finder;
	}
	
	public static Finder getPluginCompatibilityFinder(Integer parentId, Integer childId) {

		Finder finder = new Finder();

		finder.setCriteria("parentId", parentId);
		finder.setCriteria("childId", childId);

		return finder;
	}

	public static Finder getUnitFinder(Integer resourceId, String name,
			String type) {

		Finder finder = new Finder();

		finder.setCriteria("resourceId", resourceId);
		finder.setCriteria("name", name);
		finder.setCriteria("type", type);

		return finder;
	}

	public static Finder getQueuesFinder(Integer fkResourceId) {

		Finder finder = new Finder();

		finder.setCriteria("fkResourceId", fkResourceId);

		return finder;
	}

	public static Finder getExperimentStatsFinder(Integer id) {

		Finder finder = new Finder();

		finder.setCriteria("id", id);

		return finder;
	}

	public static Finder getResourceEnergyTimelineFinder(Integer fkResourceId,
			Timestamp minTimestamp, Timestamp maxTimestamp) {

		return getResourceAllocationTimelineFinder(fkResourceId, minTimestamp,
				maxTimestamp);
	}

	public static Finder getResourceAllocationTimelineFinder(
			Integer fkResourceId, Timestamp minTimestamp, Timestamp maxTimestamp) {

		Finder finder = new Finder();

		finder.setCriteria("fkResourceId", fkResourceId);
		finder.setCriteria("minTimestamp", minTimestamp);
		finder.setCriteria("maxTimestamp", maxTimestamp);

		return finder;
	}

	public static Finder getUnitEnergyTimelineFinder(Integer fkUnitId,
			Timestamp minTimestamp, Timestamp maxTimestamp) {

		return getUnitAllocationTimelineFinder(fkUnitId, minTimestamp,
				maxTimestamp);
	}

	public static Finder getUnitAllocationTimelineFinder(Integer fkUnitId,
			Timestamp minTimestamp, Timestamp maxTimestamp) {

		Finder finder = new Finder();

		finder.setCriteria("fkUnitId", fkUnitId);
		finder.setCriteria("minTimestamp", minTimestamp);
		finder.setCriteria("maxTimestamp", maxTimestamp);

		return finder;
	}

	public static Finder getUnitDetailsFinder(Integer fkUnitId) {

		Finder finder = new Finder();

		finder.setCriteria("fkUnitId", fkUnitId);

		return finder;
	}

	public static Finder getAllocationDetailsFinder(Integer fkAllocationId) {

		Finder finder = new Finder();

		finder.setCriteria("fkAllocationId", fkAllocationId);

		return finder;
	}

	public static Finder getReservationDetailsFinder(Integer fkReservationId) {

		Finder finder = new Finder();

		finder.setCriteria("fkReservationId", fkReservationId);

		return finder;
	}

	public static Finder getBlobPropertyFinder(Integer configId) {

		return getTextPropertyFinder(configId);
	}

	public static Finder getTextPropertyFinder(Integer configId) {

		Finder finder = new Finder();

		finder.setCriteria("configId", configId);

		return finder;
	}

	public String getString(String key) {

		if (map.containsKey(key)) {
			Object result = map.get(key);
			if (result instanceof String)
				return (String) result;
		}

		return null;
	}

	public Timestamp getMinTimestamp(String key) {

		if (map.containsKey(key)) {
			Object result = map.get(key);
			if (result instanceof Timestamp)
				return (Timestamp) result;
		}

		return new Timestamp(0);
	}

	public Timestamp getMaxTimestamp(String key) {

		if (map.containsKey(key)) {
			Object result = map.get(key);
			if (result instanceof Timestamp)
				return (Timestamp) result;
		}

		Calendar c = Calendar.getInstance();
		c.set(Calendar.YEAR, 2038);
		c.set(Calendar.MONTH, 0);
		c.set(Calendar.DAY_OF_MONTH, 19);
		c.set(Calendar.HOUR_OF_DAY, 3);
		c.set(Calendar.MINUTE, 14);
		c.set(Calendar.SECOND, 7);
		c.set(Calendar.MILLISECOND, 0);

		return new Timestamp(c.getTimeInMillis());
	}

	public Boolean getBoolean(String key) {

		if (map.containsKey(key)) {
			Object result = map.get(key);
			if (result instanceof Boolean)
				return (Boolean) result;
		}

		return null;
	}

	public Integer getInteger(String key) {

		if (map.containsKey(key)) {
			Object result = map.get(key);
			if (result instanceof Integer)
				return (Integer) result;
		}

		return null;
	}

	public Integer getMinInteger(String key) {

		if (map.containsKey(key)) {
			Object result = map.get(key);
			if (result instanceof Integer)
				return (Integer) result;
		}

		return Integer.MIN_VALUE;
	}

	public Integer getMaxInteger(String key) {

		if (map.containsKey(key)) {
			Object result = map.get(key);
			if (result instanceof Integer)
				return (Integer) result;
		}

		return Integer.MAX_VALUE;
	}

	public void setCriteria(String key, Object value) {
		map.put(key, value);
	}

}
