package gssim.db.mysql;

import gssim.db.dao.data.PluginCompatibility;
import gssim.db.find.Finder;
import gssim.db.utils.PreparedStatementAdapter;
import gssim.db.utils.ResultSetAdapter;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.LogFactory;

public class PluginCompatibilityDAO extends AbstractDAO implements
		gssim.db.dao.PluginCompatibilityDAO {

	protected PluginCompatibilityDAO(Connection conn) throws SQLException {
		super(conn);
		log = LogFactory.getLog(PluginDAO.class);
		setRelationName("plugin_compatibility");
	}

	@Override
	protected PreparedStatementAdapter createDeleteStmt() throws SQLException {
		String sql = "DELETE FROM plugin_compatibility WHERE parent_id = ? AND child_id = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createFindStmt() throws SQLException {

		String sql = "SELECT * FROM plugin_compatibility WHERE (parent_id = ? OR ISNULL(?))"
				+ " AND (child_id = ? OR ISNULL(?))";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createInsertStmt() throws SQLException {
		String sql = "INSERT INTO plugin_compatibility(parent_id, child_id"
				+ ") VALUES(?, ?)";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql,
						Statement.RETURN_GENERATED_KEYS));

		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createSelectStmt() throws SQLException {
		String sql = "SELECT * FROM plugin_compatibility WHERE parent_id = ? AND child_id = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createUpdateStmt() throws SQLException {
		String sql = "UPDATE plugin_compatibility SET parent_id = ?, child_id = ?"
				+ " WHERE parent_id = ? AND child_id = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	public boolean delete(PluginCompatibility obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getDeleteStmt();
		stmt.setInt(1, obj.getParentId());
		stmt.setInt(2, obj.getChildId());

		int result = stmt.executeUpdate();

		if (result == 1) {
			if (commit)
				commit();
			logSuccess("delete");
		} else
			logError("delete", result, obj);

		return (result == 1);
	}

	@Override
	public void fillObjectFromResultSet(PluginCompatibility obj, ResultSet resultSet)
			throws SQLException {

		ResultSetAdapter result = (ResultSetAdapter) resultSet;

		Integer parentId = result.getIntObj("parent_id");
		Integer childId = result.getIntObj("child_id");

		obj.setParentId(parentId);
		obj.setChildId(childId);
	}

	@Override
	public int fillStatementWithObject(PreparedStatement pstmt, PluginCompatibility obj,
			int offset) throws SQLException {

		PreparedStatementAdapter stmt = (PreparedStatementAdapter) pstmt;

		stmt.setInt(offset + 1, obj.getParentId());
		stmt.setInt(offset + 2, obj.getChildId());

		return 2;

	}

	@Override
	public List<PluginCompatibility> find(Finder finder) throws SQLException {

		PreparedStatementAdapter stmt = getFindStmt();
		List<PluginCompatibility> list = new ArrayList<PluginCompatibility>();

		stmt.setInt(1, finder.getInteger("parentId"));
		stmt.setInt(2, finder.getInteger("parentId"));
		stmt.setInt(3, finder.getInteger("childId"));
		stmt.setInt(4, finder.getInteger("childId"));

		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		while (result.next()) {
			PluginCompatibility obj = new PluginCompatibility();
			fillObjectFromResultSet(obj, result);
			list.add(obj);
		}

		return list;

	}

	@Override
	public int insert(PluginCompatibility obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		fillStatementWithObject(stmt, obj, 0);

		int result = stmt.executeUpdate();

		if (result == 1) {
			//ResultSet gk = stmt.getGeneratedKeys();
			//Integer pK = gk.next() ? gk.getInt(1) : -1;
			//obj.setPK(pK);

			if (commit)
				commit();

			logSuccess("insert");

		} else
			logError("insert", result, obj);

		return result;
	}

	public int[] insertBatch(List<PluginCompatibility> objList, boolean commit)
			throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		for (int i = 0; i < objList.size(); i++) {
			fillStatementWithObject(stmt, objList.get(i), 0);
			stmt.addBatch();
		}

		int[] result = stmt.executeBatch();

		stmt.clearBatch();

		int j = 0;

		while (j < result.length)
			if (result[j] == 1)
				j++;

		if (j == result.length) {

			//ResultSet gk = stmt.getGeneratedKeys();
			//for (int i = 0; i < objList.size(); i++) {
				//Integer PK = gk.next() ? gk.getInt(1) : -1;
				//objList.get(i).setPK(PK);
			//}

			logSuccess("batch insert", j);

			if (commit)
				commit();

		} else
			logError("batch insert", j);

		return result;
	}

	@Override
	public PluginCompatibility select(PluginCompatibility obj) throws SQLException {

		PreparedStatementAdapter stmt = getSelectStmt();
		stmt.setInt(1, obj.getParentId());
		stmt.setInt(1, obj.getChildId());
		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		result.next();

		boolean first = result.isFirst();
		boolean last = result.isLast();

		if (first && last) {

			fillObjectFromResultSet(obj, result);
			return obj;
		}

		// no user found with specified id
		if (!first && !last) {
			return null;
		}

		// no idea how this may happen (only if id is not primary key)
		throw new SQLException("Select operation in relation "
				+ getRelationName() + " returned more than one record");

	}

	@Override
	public boolean update(PluginCompatibility obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getUpdateStmt();

		int offset = fillStatementWithObject(stmt, obj, 0);

		stmt.setInt(offset + 1, obj.getParentId());
		stmt.setInt(offset + 2, obj.getChildId());

		int result = stmt.executeUpdate();

		if (result == 1) {
			if (commit)
				commit();
			logSuccess("update");
		} else
			logError("update", result, obj);

		return (result == 1);
	}

}
