package gssim.db.mysql;

import gssim.db.dao.data.User;
import gssim.db.find.Finder;
import gssim.db.utils.PreparedStatementAdapter;
import gssim.db.utils.ResultSetAdapter;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.LogFactory;

import com.mysql.jdbc.NotImplemented;

public class UserDAO extends AbstractDAO implements gssim.db.dao.UserDAO {

	protected UserDAO(Connection conn) throws SQLException {
		super(conn);
		log = LogFactory.getLog(UserDAO.class);
		setRelationName("user");
	}

	@Override
	protected PreparedStatementAdapter createDeleteStmt() throws SQLException {
		String sql = "DELETE FROM user WHERE id = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createFindStmt() throws SQLException {
		String sql = "SELECT * FROM  user WHERE dn = ? OR ISNULL(?)";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createInsertStmt() throws SQLException {
		String sql = "INSERT INTO user(dn) VALUES(?)";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql,
						Statement.RETURN_GENERATED_KEYS));

		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createSelectStmt() throws SQLException {
		String sql = "SELECT dn FROM  user WHERE id = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createUpdateStmt() throws SQLException {
		String sql = "UPDATE user SET dn = ? WHERE id = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	public boolean delete(User obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getDeleteStmt();
		stmt.setInt(1, obj.getId());

		int result = stmt.executeUpdate();

		if (result == 1) {
			if (commit)
				commit();
			logSuccess("delete");
		} else
			logError("delete", result, obj);

		return (result == 1);
	}

	@Override
	public void fillObjectFromResultSet(User obj, ResultSet result)
			throws SQLException {
		throw new NotImplemented();

	}

	@Override
	public int fillStatementWithObject(PreparedStatement stmt, User obj,
			int offset) throws SQLException {
		throw new NotImplemented();

	}

	@Override
	public List<User> find(Finder finder) throws SQLException {

		PreparedStatementAdapter stmt = getFindStmt();
		List<User> list = new ArrayList<User>();

		stmt.setString(1, finder.getString("dn"));
		stmt.setString(2, finder.getString("dn"));

		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		while (result.next()) {
			User obj = new User();

			Integer id = result.getIntObj("id");
			String dn = result.getString("dn");

			obj.setId(id);
			obj.setDN(dn);

			list.add(obj);
		}

		return list;

	}

	@Override
	public int insert(User obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();
		stmt.setString(1, obj.getDN());
		int result = stmt.executeUpdate();

		if (result == 1) {
			logSuccess("insert");

			ResultSet gk = stmt.getGeneratedKeys();
			int id = gk.next() ? gk.getInt(1) : -1;
			obj.setId(id);

		} else
			logError("insert", result, obj);

		if (commit)
			commit();

		return result;
	}

	@Override
	public int[] insertBatch(List<User> objList, boolean commit)
			throws SQLException {
		throw new NotImplemented();
	}

	@Override
	public User select(User obj) throws SQLException {

		PreparedStatementAdapter stmt = getSelectStmt();
		stmt.setInt(1, obj.getId());
		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		result.next();
		boolean first = result.isFirst();
		boolean last = result.isLast();

		if (first && last) {
			String dn = result.getString(1);
			obj.setDN(dn);
			return obj;
		}

		// no user found with specified id
		if (!first && !last) {
			return null;
		}

		// no idea how this may happen (only if id is not primary key)
		throw new SQLException("Select operation in relation "
				+ getRelationName() + " returned more than one record");

	}

	@Override
	public boolean update(User obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getUpdateStmt();
		stmt.setString(1, obj.getDN());
		stmt.setInt(2, obj.getId());

		int result = stmt.executeUpdate();

		if (result == 1) {
			if (commit)
				commit();
			logSuccess("update");
		} else
			logError("update", result, obj);

		return (result == 1);
	}

}
